#include "generic.h"
#include "vektorr2.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include "SDL/SDL_gfxPrimitives.h"

const int NO_KNOTS = 5;

VektorR2 P[NO_KNOTS];

inline void SetPixelColor(SDL_Surface *screen, int x, int y, unsigned char red, unsigned char green, unsigned char blue)
{
  ((unsigned char*)(screen->pixels))[x*3+0 + y*screen->pitch] = red;
  ((unsigned char*)(screen->pixels))[x*3+1 + y*screen->pitch] = green;
  ((unsigned char*)(screen->pixels))[x*3+2 + y*screen->pitch] = blue;
} // SetPixelColor

inline void SetPixel(SDL_Surface* screen, int x, int y)
{
  SetPixelColor(screen, x, y, 255, 255, 255);
} // SetPixel

void DisplayCurve(SDL_Surface *screen, int index)
{
  int      no_segments = 30;
  VektorR2 K, start = P[0];
  double   param;
  VektorR2 a_1, b_1, c_1, d_1, point, old_point;
  VektorR2 a_2, b_2, c_2, d_2;

  VektorR2 R[2];

  R[0] = P[0] - P[1];
  R[1] = P[4] - P[3];

  a_1 = 1.25*P[1] - 1.25*P[2] + 0.75*R[0] - 0.75*P[2] + 0.75*P[3] - 0.25*R[1];
  b_1 = -2.25*P[1] + 2.25*P[2] - 1.75*R[0] + 0.75*P[2] - 0.75*P[3] +  0.25*R[1];
  c_1 = R[0];
  d_1 = P[1];
  a_2 = -0.75*P[1] + 0.75*P[2] - 0.25*R[0] + 1.25*P[2] - 1.25*P[3] + 0.75*R[1];
  b_2 =  1.5*P[1] - 1.5*P[2] +  0.5*R[0] - 1.5*P[2] + 1.5*P[3] - 0.5*R[1];
  c_2 = -0.75*P[1] + 0.75*P[2] - 0.25*R[0] - 0.75*P[2] + 0.75*P[3] - 0.25*R[1];
  d_2 = P[2];
  
  for(double t = 0.0; t <= 2.0; t += 0.025) {
    
    if(t < 1.0) {
      point = a_1*t*t*t + b_1*t*t + c_1*t + d_1;
    } // if
    else {

      double s = t-1.0;

      point = a_2*s*s*s + b_2*s*s + c_2*s + d_2;
    } // else

    if(t > 0.0) aalineColor(screen, (int)old_point[0], (int)old_point[1], (int)point[0], (int)point[1], 0xFFFFFFFF); 

    old_point = point;

  } // for 

} // DisplayCurve

void DisplayKnots(SDL_Surface *screen)
{
  for(int i = 0; i < NO_KNOTS; i++) {
    aalineColor(screen, (int)(P[i][0])-2, (int)(P[i][1])-2, (int)(P[i][0])+2, (int)(P[i][1])+2, 0xFFFFFFFF);
    aalineColor(screen, (int)(P[i][0])+2, (int)(P[i][1])-2, (int)(P[i][0])-2, (int)(P[i][1])+2, 0xFFFFFFFF);
    //    SetPixel(screen, (int)(P[i][0]), (int)(P[i][1]));
  } // for
  
  DisplayCurve(screen, NO_KNOTS);
} // DisplayKnots

int IndexClosestPoint(int x, int y)
{
  double dist, min_dist = 100000.0;
  int    index;

  for(int i = 0; i < NO_KNOTS; i++) {
    dist = (VektorR2((float)x, (float)y) - P[i]).Norm2();

    if(min_dist > dist) {
      index = i;
      min_dist = dist;
    } // if
  } // for

  return index;
} // IndexClosestPoint

int main(int argc, char** argv)
{
    SDL_Surface      *screen, *backup_screen;
    SDL_Event        event;
    SDL_Rect         dst_rect, full_image_rect, scrolled_rect, location;
    int              window_width, window_height, thumb_height, thumb_border_size;
    long             pixel_set = 0, index;
    bool             l_button_down = false;

    window_width = 512; window_height = 512;

    P[0] = VektorR2(50, 250);
    P[1] = VektorR2(70, 50);
    P[2] = VektorR2(250, 50);
    P[3] = VektorR2(300, 230);
    P[4] = VektorR2(350, 210);

    if(SDL_Init(SDL_INIT_VIDEO) < 0) {
	cerr << "Could not initialize SDL video" << endl;
	exit(-1);
    } // if
    atexit(SDL_Quit);

    Uint32 flags = 0;

    // flags |= SDL_FULLSCREEN;

    screen = SDL_SetVideoMode(window_width, window_height, 24, flags);

    if(!screen) {
	cerr << "could not SetVideoMode() - abort" << endl;
	exit(-1);
    } // if

    DisplayKnots(screen);
   
    int min_x, max_x, min_y, max_y;
    int old_min_x, old_max_x, old_min_y, old_max_y;

    old_min_x = 4; old_max_x = window_width-4; old_min_y = 4; old_max_y = window_height-4;

    for(;;) {
        
      if((SDL_WaitEvent(&event) == 0) && (event.type == SDL_KEYDOWN)) 
	goto exit_app;

      switch(event.type) {
	
      case SDL_MOUSEMOTION:	      
	if(l_button_down) {
	  index = IndexClosestPoint(event.motion.x, event.motion.y);
	  P[index][0] = event.motion.x;
	  P[index][1] = event.motion.y;

	  bzero(screen->pixels, window_width*window_width*3);

	  min_x = 100000; max_x = -1; min_y = 100000; max_y = -1;

	  for(int i = 0; i < NO_KNOTS; i++) {
	    if(min_x > (int)P[i][0]) min_x = (int)P[i][0];
	    if(min_y > (int)P[i][1]) min_y = (int)P[i][1];

	    if(max_x < (int)P[i][0]) max_x = (int)P[i][0];
	    if(max_y < (int)P[i][1]) max_y = (int)P[i][1];
	  } // if

	  DisplayKnots(screen);
	  SDL_UnlockSurface(screen);
	  SDL_UpdateRect(screen, 0, 0, window_width, window_height);
	  SDL_LockSurface(screen);

	  old_min_x = min_x;
	  old_min_y = min_y;

	  old_max_x = max_x;
	  old_max_y = max_y;

	} // if
	break;
	
      case SDL_MOUSEBUTTONDOWN:
	l_button_down = true;
	break;
	
      case SDL_MOUSEBUTTONUP:
	l_button_down = false;
	break;
	
      case SDL_KEYDOWN:
	
	switch(event.key.keysym.sym) {
	  
	case 27:
	case 'q':
	case 'Q':
	  goto exit_app;
	  
	default:
	  break;
	} // switch
      } // switch
      
    } // for
	    
 exit_app:

    SDL_UnlockSurface(screen);	    
    SDL_FreeSurface(screen);

    exit(0);
} // main
