#ifndef _VektorR2_h
#define _VektorR2_h
#include <iostream.h>
#include <math.h> 

class VektorR2
{
private:
		float c[2]; // Komponenten des zweidim. Vektors
		
public:
		VektorR2() {}
		VektorR2(float x, float y) {c[0] = x; c[1] = y;}
		VektorR2(const VektorR2& Vektor) {c[0] = Vektor.c[0]; c[1] = Vektor.c[1];}

		float&		operator [] (const int); 
		float   	operator [] (const int) const;
		const bool	operator == (const VektorR2&) const;
		VektorR2        operator +  (const VektorR2&) const;
		VektorR2&       operator += (const VektorR2&);
		VektorR2        operator -  (const VektorR2&) const;
		VektorR2&       operator -= (const VektorR2&);
		float           operator *  (const VektorR2&) const;
		friend VektorR2 operator *  (const float, const VektorR2&);
		VektorR2        operator *  (const float);
		VektorR2& 	operator *= (const float);
		VektorR2	operator /  (const float) const;
		VektorR2&	operator /= (const float);
		static const VektorR2 CrossProduct(VektorR2&, VektorR2&);
		
		float 		Norm2() const;
		void		Normalize();

		VektorR2	Rotate(const float) const;
}; // class VektorR2

// Inline-Elementfunktionen von VektorR2

inline float& VektorR2::operator [] (const int c_index)
{
  return(c[c_index]);
} // VektorR2::operator []

inline float VektorR2::operator [] (const int c_index) const
{
	return(c[c_index]);
} // VektorR2::operator []

inline const bool VektorR2::operator == (const VektorR2& Vektor) const
{
	return((c[0] == Vektor.c[0]) && (c[1] == Vektor.c[1]));
} // VektorR2::operator ==

inline VektorR2 VektorR2::operator + (const VektorR2& Vektor) const
{
	return VektorR2(c[0] + Vektor.c[0], c[1] + Vektor.c[1]);
} // VektorR2::operator+

inline VektorR2& VektorR2::operator += (const VektorR2& Vektor)
{
	c[0] += Vektor.c[0];
	c[1] += Vektor.c[1];
	
	return *this;
} // VektorR2::operator +=

inline VektorR2 VektorR2::operator - (const VektorR2& Vektor) const
{
	return VektorR2(c[0] - Vektor.c[0], c[1] - Vektor.c[1]);
} // VektorR2::operator -

inline VektorR2& VektorR2::operator -= (const VektorR2& Vektor)
{
	c[0] -= Vektor.c[0];
	c[1] -= Vektor.c[1];
	
	return *this;
} // VektorR2::operator -=

inline float VektorR2::operator * (const VektorR2& Vektor) const
{
	return (c[0]*Vektor.c[0] + c[1]*Vektor.c[1]);
} // VektorR2::operator *

inline VektorR2 operator * (const float s, const VektorR2& Vektor)
{
	return VektorR2(s*Vektor.c[0], s*Vektor.c[1]);
} // friend operator * (const float, const VektorR2&);

inline VektorR2 VektorR2::operator * (const float s)
{
	return VektorR2(s*c[0], s*c[1]);
} // VektorR2::operator * (const float, const VektorR2&);

inline VektorR2& VektorR2::operator *= (const float s)
{
	c[0] *= s;
	c[1] *= s;
	
	return *this;
} // VektorR2::opertor *= (const float&)

inline VektorR2 VektorR2::operator / (const float s) const
{
	return VektorR2(c[0]/s, c[1]/s);
} // VektorR2::operator /

inline VektorR2& VektorR2::operator /= (const float s)
{
	c[0] /= s;
	c[1] /= s;
	
	return *this;
} // VektorR2::operator /=

inline float VektorR2::Norm2() const
{
	return ((float)(sqrt((double)(c[0]*c[0] + c[1]*c[1]))));
} // VektorR2::Norm2

inline void VektorR2::Normalize() {
	float length = Norm2();

	if(length > 0.0) *this /= length;
} // VektorR2::Normalize

inline VektorR2 VektorR2::Rotate(const float angle) const {
	
	double   current_angle;
	float    length;
	VektorR2 v = *this;

	length = v.Norm2();
	v /= length;

	current_angle = acos((double)v[0]);
	if(v[1] < 0.0) current_angle = 2.0*M_PI - current_angle;
	current_angle += (double)angle;

	return length*VektorR2((float)cos(current_angle), (float)sin(current_angle));
} // VektorR2::Rotate
#endif
